﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DDStoDXT.Types
{
    public struct FourCC
    {
        private readonly uint value;

        private static readonly char[] VALID_CHARS = {'_',' ','0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','N','O','P','Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z'};

        public uint NumericValue
        {
            get
            {
                return value;
            }
        }

        public string Name
        {
            get
            {
                return Encoding.ASCII.GetString(BitConverter.GetBytes(value));
            }
        }

        public bool IsValid
        {
            get
            {
                string name = this.Name;
                for (int i = 0; i < 4; i++)
                {
                    if (!VALID_CHARS.Contains(name[i]))
                    {
                        return false;
                    }
                }
                return true;
            }
        }

        public FourCC(uint value)
        {
            this.value = value;
        }

        public FourCC(string value)
        {
            this.value = BitConverter.ToUInt32(Encoding.ASCII.GetBytes(value), 0);
        }

        public static bool operator ==(FourCC a, FourCC b)
        {
            return a.value == b.value;
        }

        public static bool operator !=(FourCC a, FourCC b)
        {
            return a.value != b.value;
        }

        public static implicit operator FourCC(string a)
        {
            if (a.Length != 4)
            {
                throw new ArgumentException("FourCC string must be 4 characters long");
            }
            return new FourCC(a);
        }

        public override int GetHashCode()
        {
            return value.GetHashCode();
        }

        public override string ToString()
        {
            return Name;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;

            if (!(obj is FourCC))
            {
                return false;
            }
            
            return ((FourCC) obj).value == this.value;
        }
    }
}
