﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using DDStoDXT.IO;
using DDStoDXT.Types;

namespace DDStoDXT
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnAddFiles_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog dlg = new OpenFileDialog())
            {
                dlg.Filter = "DirectDraw Surface files (*.dds)|*.dds|All files|*.*";
                dlg.FilterIndex = 0;
                dlg.Title = "Select files to add to conversion list";
                dlg.Multiselect = true;
                if (dlg.ShowDialog(this) == DialogResult.OK)
                {
                    listFiles.Items.AddRange(dlg.FileNames);
                }
            }
        }

        private void btnConvert_Click(object sender, EventArgs e)
        {
            foreach (object obj in listFiles.Items)
            {
                string path = obj as string;
                if (!String.IsNullOrEmpty(path) && File.Exists(path))
                {
                    try
                    {
                        ConvertFile(path);
                    }
                    catch (Exception ex)
                    {
                        Log("ERROR: {0}", ex.Message);
                    }
                }
            }
        }

        private void Log(string message, params object[] args)
        {
            edLog.AppendText(String.Format(message, args));
            edLog.AppendText(Environment.NewLine);
            edLog.Select(edLog.Text.Length, edLog.Text.Length);
            edLog.ScrollToCaret();
        }

        private void ConvertFile(string path)
        {
            string outputPath = Path.ChangeExtension(path, ".dxt");
            using (FileReader reader = new FileReader(path))
            {
                FourCC fourcc = reader.ReadFourCC();
                if (fourcc != "DDS ")
                {
                    Log("Invalid DDS format (no DDS magic)");
                    return;
                }

                uint recordSize = reader.ReadU32LE();
                if (recordSize != 124)
                {
                    Log("Invalid DDS format (wrong record size, probably DirectX 10 format?)");
                    return;
                }

                uint flags = reader.ReadU32LE();

                uint height = reader.ReadU32LE();
                uint width = reader.ReadU32LE();
                reader.Position = 80;
                uint pixelFormat = reader.ReadU32LE();
                if ((pixelFormat & 0x00000004) == 0)
                {
                    Log("Wrong pixel format (must be FOURCC)");
                    return;
                }

                FourCC fourCC = reader.ReadFourCC();
                switch (fourCC.Name)
                {
                    case "DXT1":
                    case "DXT5":
                        break;
                    case "DXT2":
                    case "DXT3":
                    case "DXT4":
                        Log("Warning: Compression {0} detected. It is unknown if SOMI:SE supports this.");
                        break;
                    default:
                        Log("Warning: Unknown compression: {0}. Not sure if this will work.");
                        break;
                }

                reader.Position = 128;

                Log("Writing {0}...", outputPath);

                using (FileWriter writer = new FileWriter(outputPath))
                {
                    writer.WriteFourCC(fourCC);
                    writer.WriteU32LE(width);
                    writer.WriteU32LE(height);

                    byte[] buffer = new byte[32000];
                    while (true)
                    {

                        int read = reader.Read(buffer, 0, buffer.Length);
                        if (read == 0)
                        {
                            break;
                        }
                        writer.Write(read, buffer);
                    }
                }
            }
        }
    }
}
